# Python 2.7
import socket
import RPi.GPIO as GPIO
import wiringpi
import spidev

def dac_write(DAC_channel, DAC_value):
    spi.open(0,1)
    bin_data_bits = int(bin(DAC_value)[2:])
    bin_data_bits = "{0:08d}".format(bin_data_bits)
    whole_list = [DAC_channel, 0, 1, 1, bin_data_bits, '0000']
    final_bitstring = ''.join( map( str, whole_list ) )
    byte1 = int(final_bitstring[0:8], 2)
    byte2 = int(final_bitstring[8:16], 2)
    r = spi.xfer2([byte1, byte2])
    spi.close()

def get_adc(channel):
    spi.open(0,0)
    r = spi.xfer2([1, (2+channel)<<6, 0])
    ret = ((r[1]&31) << 6) + (r[2] >> 2)
    spi.close()
    return ret

UDP_IP = "192.168.002.105"
UDP_PORT = 1600
sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
sock.bind((UDP_IP, UDP_PORT))
GPIO.setwarnings(False)
GPIO.setmode(GPIO.BCM)
wiringpi.wiringPiSetupGpio()
spi = spidev.SpiDev()
while True:
    data, adr = sock.recvfrom(1024)
    # Inputs
    # Format: RInn (Read input nn)
    #         RDnn (Read input nn (internal pull-down))
    #         RUnn (Read input nn (internal pull-up))
    #         ==> Raspberry answers "0" or "1"
    #         RAn (Read analog channel n)
    #         ==> Raspberry sends 3-byte-value of analog input channel
    if (data[0] == "R"):
        if data[1] == "I":
            i = int(data[2]+data[3])
            GPIO.setup(i, GPIO.IN)
        if data[1] == "D":
            i = int(data[2]+data[3])
            GPIO.setup(i, GPIO.IN, pull_up_down = GPIO.PUD_DOWN)
        if data[1] == "U":
            i = int(data[2]+data[3])
            GPIO.setup(i, GPIO.IN, pull_up_down = GPIO.PUD_UP)
        if ((data[1] == "I") or (data[1] == "D") or (data[1] == "U")):
            n = GPIO.input(i)
            if (n == 1):
                sock.sendto("1", adr)
            if (n == 0):
                sock.sendto("0", adr)
        if data[1] == "A":
            ch = int(data[2])
            s = str(get_adc(ch))
            sock.sendto(s, adr)
    # Outputs
    # Format: WOnn:v    (Write value v to output nn, v = "0" or "1")
    #         WPnn:vvvv (Write value vvvv to PWM output nn, vvvv = 0 ... 1023)
    #         WAn:vvv   (Write value vvv to analog output channel n, vvv = 0 ... 255)
    #         Raspberry answers "OK"
    if (data[0] == "W"):
        if data[1] == "O":
            i = int(data[2]+data[3])
            v = int(data[5])
            GPIO.setup(i, GPIO.OUT)
            if (v == 0):
                GPIO.output(i, GPIO.LOW)
            if (v == 1):
                GPIO.output(i, GPIO.HIGH)
        if data[1] == "P":
            i = int(data[2]+data[3])
            v = int(data[5]+data[6]+data[7]+data[8])
            wiringpi.pinMode(i, 2)
            wiringpi.pwmWrite(i, v)
        if data[1] == "A":
            ch = int(data[2])
            v = int(data[4]+data[5]+data[6])
            dac_write(ch, v)
        sock.sendto("OK", adr)
