Library RaspberryGert;

// RaspberryPi Driver for WinFACT (all versions)
// DELHPI 6

{$R *.res}

uses
  WinProcs,
  WinTypes,
  SysUtils,
  Controls,
  Forms,
  Classes,
  Graphics,
  StdCtrls,
  Dialogs,
  Sockets,
  NMUDP,
  Math,
  RaspberryGertDlg in 'RaspberryGertDlg.pas' {RaspberryGertDialog};

const GPIOPINS: array[0..7] of integer = (4, 17, 18, 22, 23, 24, 25, 27);

type
  TDummy = class  // Dummy Class to receive UDP Data
    OwnerUserData: Pointer;
    procedure DataReceived(Sender: TComponent; NumberBytes: Integer; FromIP: string; Port: integer);
  end;

  PUserData = ^TUserData;
  TUserData = record
    Dummy: TDummy;          // Dummy Class to receive UDP Data
    DataReceived: boolean;  // Data received Flag
    ReceivedString: string; // Received String
    UDPClient: TNMUDP;      // Client for UDP Communication
    // Configuration data
    RemotePort: integer;  // Remote-Port for UDP
    IPAddress: string;    // IP-Address of Arduino
    IOMode: array[0..7] of integer; // 0: unused, 1: Input, 2: Output
    ADUsed, DAUsed: array[0..1] of integer;
  end;

  PParameterStruct=^TParameterStruct;
  TParameterStruct=packed record
    NuE : Byte;
    NuI : Byte;
    NuB : Byte;
    E:Array[0..31] of Extended;
    I:Array[0..31] of Integer;
    B:Array[0..31] of Byte;
    D:Array[0..255] of char;
    EMin:Array[0..31] of Extended;
    EMax:Array[0..31] of Extended;
    IMin:Array[0..31] of Integer;
    IMax:Array[0..31] of Integer;
    NaE : Array[0..31,0..40] of char;
    NaI : Array[0..31,0..40] of char;
    NaB : Array[0..31,0..40] of char;
    UserDataPtr: PUserData;
    ParentPtr: Pointer;
    ParentHWnd: HWnd;
    ParentName: PChar;
    UserHWindow: HWnd;
    DataFile: text;
  end;

  PDialogEnableStruct=^TDialogEnableStruct;
  TDialogEnablestruct=packed record
    AllowE: Longint;
    AllowI: Longint;
    AllowB: Longint;
    AllowD: Byte;
  end;

  PNumberOfInputsOutputs=^TNumberOfInputsOutputs;
  TNumberOfInputsOutputs=packed record
    Inputs :Byte;
    Outputs:Byte;
    NameI : Array[0..49,0..40] of char;
    NameO : Array[0..49,0..40] of char;
  end;

  PInputArray = ^TInputArray;
  TInputArray = packed array[1..30] of extended;
  POutputArray = ^TOutputArray;
  TOutputArray = packed array[1..30] of extended;

var
  COMHandle: THandle;

procedure GetParameterStruct(D:PParameterStruct);export stdcall;
begin
  D^.NuE := 0;
  D^.NuI := 1;
  D^.NuB := 0;
  D^.I[0] := 0;
end;

procedure GetDialogEnableStruct(D:PDialogEnableStruct;D2:PParameterStruct);export stdcall;
begin
  D^.AllowE:=$FFFFFFFF;
  D^.AllowB:=$FFFFFFFF;
  D^.AllowI:=$FFFFFFFF;
  D^.AllowD:= 1;
end;

procedure GetNumberOfInputsOutputs(D:PNumberOfInputsOutputs);export stdcall;
begin
  D^.Outputs := 10;
  D^.Inputs := 10;
  // Outputs
  StrPCopy(D^.NameO[0], 'GPIO Pin 4');
  StrPCopy(D^.NameO[1], 'GPIO Pin 17');
  StrPCopy(D^.NameO[2], 'GPIO Pin 18');
  StrPCopy(D^.NameO[3], 'GPIO Pin 22');
  StrPCopy(D^.NameO[4], 'GPIO Pin 23');
  StrPCopy(D^.NameO[5], 'GPIO Pin 24');
  StrPCopy(D^.NameO[6], 'GPIO Pin 25');
  StrPCopy(D^.NameO[7], 'GPIO Pin 27');
  StrPCopy(D^.NameO[8], 'AD0');
  StrPCopy(D^.NameO[9], 'AD1');
  // Inputs
  StrPCopy(D^.NameI[0], 'GPIO Pin 4');
  StrPCopy(D^.NameI[1], 'GPIO Pin 17');
  StrPCopy(D^.NameI[2], 'GPIO Pin 18');
  StrPCopy(D^.NameI[3], 'GPIO Pin 22');
  StrPCopy(D^.NameI[4], 'GPIO Pin 23');
  StrPCopy(D^.NameI[5], 'GPIO Pin 24');
  StrPCopy(D^.NameI[6], 'GPIO Pin 25');
  StrPCopy(D^.NameI[7], 'GPIO Pin 27');
  StrPCopy(D^.NameI[8], 'DA0');
  StrPCopy(D^.NameI[9], 'DA1');
end;

function SetOutputChar: PChar; export stdcall;
begin
  Result := '41&71&82&22&32&42&52&7A&0A&1';
end;

function SetInputChar: PChar; export stdcall;
begin
  Result := '41&71&82&22&32&42&52&7A&0A&1';
end;

procedure TDummy.DataReceived(Sender: TComponent; NumberBytes: Integer; FromIP: string; Port: integer);
var Buffer: array[0..300] of char;
    nReadBytes: integer;
begin
  FillChar(Buffer, sizeof(Buffer), 0);
  with PUserData(OwnerUserData)^ do begin
    UDPClient.ReadBuffer(Buffer, nReadBytes);
    ReceivedString := StrPas(Buffer);
    DataReceived := true;
  end;
end;

procedure SendString(D: PParameterStruct; Command: string);
// Send String
var Buffer: array[0..100] of char;
begin
  StrPCopy(Buffer, Command);
  D^.UserDataPtr.UDPClient.SendBuffer(Buffer, Length(Command));
end;

function ReadString(D: PParameterStruct): string;
// Read String
var i, StartTime: integer;
    BytesRead: DWORD;
    InBytes: array[1..1] of byte;
begin
  StartTime := GetTickCount;
  D^.UserDataPtr.DataReceived := false;
  D^.UserDataPtr.ReceivedString := '';
  repeat
    Application.ProcessMessages;
  until D^.UserDataPtr.DataReceived or ((GetTickCount - StartTime) > 100);
  Result := D^.UserDataPtr.ReceivedString;
end;

function CanSimulateDLL(D:PParameterStruct):Integer; export stdcall;
// Called to check if simulation is possible
begin
  Result := 1;
end;

procedure SimulateDLL(T:Extended;D1:PParameterStruct;Inputs:PInputArray;Outputs:POutputArray);export stdcall;
// Called for each simulation step
var
  i, v: integer;
  s: string;
begin
  // Digital In
  for i:=0 to 7 do if (D1^.UserDataPtr.IOMode[i] = 1) or (D1^.UserDataPtr.IOMode[i] = 3)  or (D1^.UserDataPtr.IOMode[i] = 4) then begin
    case D1^.UserDataPtr.IOMode[i] of
      1: SendString(D1, 'RI' + Format('%2d', [GPIOPINS[i]]));
      3: SendString(D1, 'RD' + Format('%2d', [GPIOPINS[i]]));
      4: SendString(D1, 'RU' + Format('%2d', [GPIOPINS[i]]));
    end;
    if ReadString(D1) = '1' then
      Outputs^[i+1] := 5.0
    else
      Outputs^[i+1] := 0.0;
  end;
  // Digital Out
  for i:=0 to 7 do if D1^.UserDataPtr.IOMode[i] = 2 then begin
    s := 'WO' + Format('%2d', [GPIOPINS[i]]) + ':';
    if Inputs^[i+1] > 2.5 then
      s := s + '1'
    else
      s := s + '0';
    SendString(D1, s);
    ReadString(D1);  // Wait for OK
  end;
  // PWM Out
  for i:=0 to 7 do if D1^.UserDataPtr.IOMode[i] = 5 then begin
    s := 'WP' + Format('%2d', [GPIOPINS[i]]) + ':';
    v := round(Max(Min(Inputs^[i+1], 1024), 0));
    s := s + Format('%4d', [v]);
    SendString(D1, s);
    ReadString(D1);  // Wait for OK
  end;
  // Analog In
  for i:=0 to 1 do if (D1^.UserDataPtr.ADUsed[i] = 1) then begin
    SendString(D1, 'RA' + Format('%1d', [i]));
    Outputs^[i+9] := StrToFloat(ReadString(D1));
  end;
  // Analog Out
  for i:=0 to 1 do if D1^.UserDataPtr.DAUsed[i] = 1 then begin
    s := 'WA' + Format('%1d', [i]) + ':';
    v := round(Max(Min(Inputs^[i+9], 255), 0));
    s := s + Format('%3d', [v]);
    SendString(D1, s);
    ReadString(D1);  // Wait for OK
  end;
end;

procedure InitSimulationDLL(D:PParameterStruct;Inputs:PInputArray;Outputs:POutputArray);export stdcall;
// Called when simulation starts
begin
  with D^.UserDataPtr^ do begin
    UDPClient.RemoteHost := IPAddress;
    UDPClient.RemotePort := RemotePort;
  end;
  SimulateDLL(0, D, Inputs, Outputs);
end;

procedure EndSimulationDLL2(D:PParameterStruct); export stdcall;
// Called when simulation was terminated
begin
end;

procedure InitUserDLL(D: PParameterStruct); export stdcall;
var i: integer;
begin
  Application.Handle := D^.ParentHWnd;
  D^.UserDataPtr := new(PUserData);
  with D^.UserDataPtr^ do begin
    IPAddress := '192.168.0.121';
    RemotePort := 1600;
    for i:=0 to 7 do
      IOMode[i] := 1;
    for i:=0 to 1 do begin
      ADUsed[i] := 0;
      DAUsed[i] := 0;
    end;
    // UDP Client
    Dummy := TDummy.Create;
    Dummy.OwnerUserData := D^.UserDataPtr;
    UDPClient := TNMUDP.Create(nil);
    UDPClient.OnDataReceived := Dummy.DataReceived;
    UDPClient.LocalPort := 1600;
  end;
end;

procedure DisposeUserDLL(D: PParameterStruct); export stdcall;
begin
  Application.Handle := 0;
  D^.UserDataPtr.UDPClient.Free;
  Dispose(D^.UserDataPtr);
end;

function GetDLLName: PChar; export stdcall;
begin
  GetDLLName := 'RaspberryPi / GERTBoard';
end;

procedure CallParameterDialogDLL(D1:PParameterStruct; D2:PNumberOfInputsOutputs);export stdcall;
var Dialog: TRaspberryGertDialog;
begin
  Dialog := TRaspberryGertDialog.Create(Application);
  with Dialog, D1^.UserDataPtr^ do begin
    IPAddressEdit.Text := IPAddress;
    PortEdit.Text := IntToStr(RemotePort);
    In0CB.ItemIndex := IOMode[0];
    In1CB.ItemIndex := IOMode[1];
    In2CB.ItemIndex := IOMode[2];
    In3CB.ItemIndex := IOMOde[3];
    In4CB.ItemIndex := IOMode[4];
    In5CB.ItemIndex := IOMode[5];
    In6CB.ItemIndex := IOMode[6];
    In7CB.ItemIndex := IOMode[7];
    AD0CB.ItemIndex := ADUsed[0];
    AD1CB.ItemIndex := ADUsed[1];
    DA0CB.ItemIndex := DAUsed[0];
    DA1CB.ItemIndex := DAUsed[1];
    if ShowModal = mrOK then begin
      IPAddress := IPAddressEdit.Text;
      try
        RemotePort := StrToInt(PortEdit.Text);
      except
      end;
      IOMode[0] := In0CB.ItemIndex;
      IOMode[1] := In1CB.ItemIndex;
      IOMode[2] := In2CB.ItemIndex;
      IOMode[3] := In3CB.ItemIndex;
      IOMode[4] := In4CB.ItemIndex;
      IOMode[5] := In5CB.ItemIndex;
      IOMode[6] := In6CB.ItemIndex;
      IOMode[7] := In7CB.ItemIndex;
      ADUsed[0] := AD0CB.ItemIndex;
      ADUsed[1] := AD1CB.ItemIndex;
      DAUsed[0] := DA0CB.ItemIndex;
      DAUsed[1] := DA1CB.ItemIndex;
    end;
  end;
  Dialog.Free;
end;

Procedure WriteToFile(AFileHandle:Word; D: PParameterStruct); export stdcall;
var i: integer;
    s: array[0..1000] of char;
begin
  with D^.UserDataPtr^ do begin
    StrPCopy(s, IPAddress + #13#10); _lWrite(AFileHandle, s, StrLen(s));
    StrPCopy(s, IntToStr(RemotePort) + #13#10); _lWrite(AFileHandle, s, StrLen(s));
    for i:=0 to 7 do begin
      StrPCopy(s, IntToStr(IOMode[i]) + #13#10); _lWrite(AFileHandle, s, StrLen(s));
    end;
    for i:=0 to 1 do begin
      StrPCopy(s, IntToStr(ADUsed[i]) + #13#10); _lWrite(AFileHandle, s, StrLen(s));
    end;
    for i:=0 to 1 do begin
      StrPCopy(s, IntToStr(DAUsed[i]) + #13#10); _lWrite(AFileHandle, s, StrLen(s));
    end;
  end;
end;

Procedure ReadFromFile(AFileHandle:Word; D: PParameterStruct); export stdcall;
var i: integer;
    s: array[0..1000] of char;
procedure ReadOneLine(FHandle:Word; Aps :PChar);
var i  : Integer;
begin
  i:=0;
  _lRead(FHandle,@Aps[i],1);
  repeat
    inc(i);
    _lRead(FHandle,@Aps[i],1);
  until (Aps[i-1]=#13) and (Aps[i]=#10);
  Aps[i-1]:=#0;
end;
begin
  ReadOneLine(AFileHandle, s); D^.UserDataPtr.IPAddress := StrPas(s);
  ReadOneLine(AFileHandle, s); D^.UserDataPtr.RemotePort := StrToInt(StrPas(s));
  for i:=0 to 7 do begin
    ReadOneLine(AFileHandle, s); D^.UserDataPtr.IOMode[i] := StrToInt(StrPas(s));
  end;
  for i:=0 to 1 do begin
    ReadOneLine(AFileHandle, s); D^.UserDataPtr.ADUsed[i] := StrToInt(StrPas(s));
  end;
  for i:=0 to 1 do begin
    ReadOneLine(AFileHandle, s); D^.UserDataPtr.DAUsed[i] := StrToInt(StrPas(s));
  end;
end;

procedure IsUserDLL32; export stdcall;
begin
end;

procedure IsDemoDLL; export stdcall;
begin
end;

exports
  GetParameterStruct,
  GetDialogEnableStruct,
  GetNumberOfInputsOutputs,
  CanSimulateDLL,
  InitSimulationDLL,
  SimulateDLL,
  InitUserDLL,
  DisposeUserDLL,
  EndSimulationDLL2,
  GetDLLName,
  CallParameterDialogDLL,
  WriteToFile,
  ReadFromFile,
  SetInputChar,
  SetOutputChar,
  IsDemoDLL,
  IsUserDLL32;


begin
  DecimalSeparator := '.';
  Application.UpdateFormatSettings := false;
end.
