Library RaspberryPiFace;

// RaspberryPi Driver for WinFACT (all versions)
// DELHPI 6

{$R *.res}

uses
  WinProcs,
  WinTypes,
  SysUtils,
  Controls,
  Forms,
  Classes,
  Graphics,
  StdCtrls,
  Dialogs,
  Sockets,
  NMUDP,
  Math,
  RaspberryPiFaceDlg in 'RaspberryPiFaceDlg.pas' {RaspberryPiFaceDialog};

type
  TDummy = class  // Dummy Class to receive UDP Data
    OwnerUserData: Pointer;
    procedure DataReceived(Sender: TComponent; NumberBytes: Integer; FromIP: string; Port: integer);
  end;

  PUserData = ^TUserData;
  TUserData = record
    Dummy: TDummy;          // Dummy Class to receive UDP Data
    DataReceived: boolean;  // Data received Flag
    ReceivedString: string; // Received String
    UDPClient: TNMUDP;      // Client for UDP Communication
    // Configuration data
    RemotePort: integer;  // Remote-Port for UDP
    IPAddress: string;    // IP-Address of Arduino
    InUsed, OutUsed: array[0..7] of integer;
  end;

  PParameterStruct=^TParameterStruct;
  TParameterStruct=packed record
    NuE : Byte;
    NuI : Byte;
    NuB : Byte;
    E:Array[0..31] of Extended;
    I:Array[0..31] of Integer;
    B:Array[0..31] of Byte;
    D:Array[0..255] of char;
    EMin:Array[0..31] of Extended;
    EMax:Array[0..31] of Extended;
    IMin:Array[0..31] of Integer;
    IMax:Array[0..31] of Integer;
    NaE : Array[0..31,0..40] of char;
    NaI : Array[0..31,0..40] of char;
    NaB : Array[0..31,0..40] of char;
    UserDataPtr: PUserData;
    ParentPtr: Pointer;
    ParentHWnd: HWnd;
    ParentName: PChar;
    UserHWindow: HWnd;
    DataFile: text;
  end;

  PDialogEnableStruct=^TDialogEnableStruct;
  TDialogEnablestruct=packed record
    AllowE: Longint;
    AllowI: Longint;
    AllowB: Longint;
    AllowD: Byte;
  end;

  PNumberOfInputsOutputs=^TNumberOfInputsOutputs;
  TNumberOfInputsOutputs=packed record
    Inputs :Byte;
    Outputs:Byte;
    NameI : Array[0..49,0..40] of char;
    NameO : Array[0..49,0..40] of char;
  end;

  PInputArray = ^TInputArray;
  TInputArray = packed array[1..30] of extended;
  POutputArray = ^TOutputArray;
  TOutputArray = packed array[1..30] of extended;

var
  COMHandle: THandle;

procedure GetParameterStruct(D:PParameterStruct);export stdcall;
begin
  D^.NuE := 0;
  D^.NuI := 1;
  D^.NuB := 0;
  D^.I[0] := 0;
end;

procedure GetDialogEnableStruct(D:PDialogEnableStruct;D2:PParameterStruct);export stdcall;
begin
  D^.AllowE:=$FFFFFFFF;
  D^.AllowB:=$FFFFFFFF;
  D^.AllowI:=$FFFFFFFF;
  D^.AllowD:= 1;
end;

procedure GetNumberOfInputsOutputs(D:PNumberOfInputsOutputs);export stdcall;
begin
  D^.Outputs := 8;
  D^.Inputs := 8;
  // Outputs
  StrPCopy(D^.NameO[0], 'In 0');
  StrPCopy(D^.NameO[1], 'In 1');
  StrPCopy(D^.NameO[2], 'In 2');
  StrPCopy(D^.NameO[3], 'In 3');
  StrPCopy(D^.NameO[4], 'In 4');
  StrPCopy(D^.NameO[5], 'In 5');
  StrPCopy(D^.NameO[6], 'In 6');
  StrPCopy(D^.NameO[7], 'In 7');
  // Inputs
  StrPCopy(D^.NameI[0], 'Out 0');
  StrPCopy(D^.NameI[1], 'Out 1');
  StrPCopy(D^.NameI[2], 'Out 2');
  StrPCopy(D^.NameI[3], 'Out 3');
  StrPCopy(D^.NameI[4], 'Out 4');
  StrPCopy(D^.NameI[5], 'Out 5');
  StrPCopy(D^.NameI[6], 'Out 6');
  StrPCopy(D^.NameI[7], 'Out 7');
end;

function SetOutputChar: PChar; export stdcall;
begin
  Result := 'I&0I&1I&2I&3I&4I&5I&6I&7';
end;

function SetInputChar: PChar; export stdcall;
begin
  Result := 'O&0O&1O&2O&3O&4O&5O&6O&7O&8';
end;

procedure TDummy.DataReceived(Sender: TComponent; NumberBytes: Integer; FromIP: string; Port: integer);
var Buffer: array[0..300] of char;
    nReadBytes: integer;
begin
  FillChar(Buffer, sizeof(Buffer), 0);
  with PUserData(OwnerUserData)^ do begin
    UDPClient.ReadBuffer(Buffer, nReadBytes);
    ReceivedString := StrPas(Buffer);
    DataReceived := true;
  end;
end;

procedure SendString(D: PParameterStruct; Command: string);
// Send String
var Buffer: array[0..100] of char;
    i: integer;
begin
  StrPCopy(Buffer, Command);
  D^.UserDataPtr.UDPClient.SendBuffer(Buffer, Length(Command));
end;

function ReadString(D: PParameterStruct): string;
var StartTime: integer;
// Read String
begin
  StartTime := GetTickCount;
  D^.UserDataPtr.DataReceived := false;
  D^.UserDataPtr.ReceivedString := '';
  repeat
    Application.ProcessMessages;
  until D^.UserDataPtr.DataReceived or ((GetTickCount - StartTime) > 100);
  Result := D^.UserDataPtr.ReceivedString;
end;

function CanSimulateDLL(D:PParameterStruct):Integer; export stdcall;
// Called to check if simulation is possible
begin
  Result := 1;
end;

procedure SimulateDLL(T:Extended;D1:PParameterStruct;Inputs:PInputArray;Outputs:POutputArray);export stdcall;
// Called for each simulation step
var
  i: integer;
  s: string;
begin
  for i:=0 to 7 do if D1^.UserDataPtr.InUsed[i] > 0 then begin
    SendString(D1, 'RI' + IntToStr(i));
    if ReadString(D1) = '1' then
      Outputs^[i+1] := 5.0
    else
      Outputs^[i+1] := 0.0;
  end;
  for i:=0 to 7 do if D1^.UserDataPtr.OutUsed[i] > 0 then begin
    s := 'WO' + IntToStr(i) + ':';
    if Inputs^[i+1] > 2.5 then
      s := s + '1'
    else
      s := s + '0';
    SendString(D1, s);
    ReadString(D1);  // Wait for OK
  end;
end;

procedure InitSimulationDLL(D:PParameterStruct;Inputs:PInputArray;Outputs:POutputArray);export stdcall;
// Called when simulation starts
begin
  with D^.UserDataPtr^ do begin
    UDPClient.RemoteHost := IPAddress;
    UDPClient.RemotePort := RemotePort;
  end;
  SimulateDLL(0, D, Inputs, Outputs);
end;

procedure EndSimulationDLL2(D:PParameterStruct); export stdcall;
// Called when simulation was terminated
begin
end;

procedure InitUserDLL(D: PParameterStruct); export stdcall;
var i: integer;
begin
  Application.Handle := D^.ParentHWnd;
  D^.UserDataPtr := new(PUserData);
  with D^.UserDataPtr^ do begin
    IPAddress := '192.168.0.121';
    RemotePort := 1600;
    for i:=0 to 7 do begin
      InUsed[i] := 1;
      OutUsed[i] := 1;
    end;  
    // UDP Client
    Dummy := TDummy.Create;
    Dummy.OwnerUserData := D^.UserDataPtr;
    UDPClient := TNMUDP.Create(nil);
    UDPClient.OnDataReceived := Dummy.DataReceived;
    UDPClient.LocalPort := 1600;
  end;
end;

procedure DisposeUserDLL(D: PParameterStruct); export stdcall;
begin
  Application.Handle := 0;
  D^.UserDataPtr.UDPClient.Free;
  Dispose(D^.UserDataPtr);
end;

function GetDLLName: PChar; export stdcall;
begin
  GetDLLName := 'RaspberryPi / PiFace Digital';
end;

procedure CallParameterDialogDLL(D1:PParameterStruct; D2:PNumberOfInputsOutputs);export stdcall;
var Dialog: TRaspberryPiFaceDialog;
begin
  Dialog := TRaspberryPiFaceDialog.Create(Application);
  with Dialog, D1^.UserDataPtr^ do begin
    IPAddressEdit.Text := IPAddress;
    PortEdit.Text := IntToStr(RemotePort);
    In0CB.ItemIndex := InUsed[0];
    In1CB.ItemIndex := InUsed[1];
    In2CB.ItemIndex := InUsed[2];
    In3CB.ItemIndex := InUsed[3];
    In4CB.ItemIndex := InUsed[4];
    In5CB.ItemIndex := InUsed[5];
    In6CB.ItemIndex := InUsed[6];
    In7CB.ItemIndex := InUsed[7];
    Out0CB.ItemIndex := OutUsed[0];
    Out1CB.ItemIndex := OutUsed[1];
    Out2CB.ItemIndex := OutUsed[2];
    Out3CB.ItemIndex := OutUsed[3];
    Out4CB.ItemIndex := OutUsed[4];
    Out5CB.ItemIndex := OutUsed[5];
    Out6CB.ItemIndex := OutUsed[6];
    Out7CB.ItemIndex := OutUsed[7];
    if ShowModal = mrOK then begin
      IPAddress := IPAddressEdit.Text;
      try
        RemotePort := StrToInt(PortEdit.Text);
      except
      end;
      InUsed[0] := In0CB.ItemIndex;
      InUsed[1] := In1CB.ItemIndex;
      InUsed[2] := In2CB.ItemIndex;
      InUsed[3] := In3CB.ItemIndex;
      InUsed[4] := In4CB.ItemIndex;
      InUsed[5] := In5CB.ItemIndex;
      InUsed[6] := In6CB.ItemIndex;
      InUsed[7] := In7CB.ItemIndex;
      OutUsed[0] := Out0CB.ItemIndex;
      OutUsed[1] := Out1CB.ItemIndex;
      OutUsed[2] := Out2CB.ItemIndex;
      OutUsed[3] := Out3CB.ItemIndex;
      OutUsed[4] := Out4CB.ItemIndex;
      OutUsed[5] := Out5CB.ItemIndex;
      OutUsed[6] := Out6CB.ItemIndex;
      OutUsed[7] := Out7CB.ItemIndex;
    end;
  end;
  Dialog.Free;
end;

Procedure WriteToFile(AFileHandle:Word; D: PParameterStruct); export stdcall;
var i: integer;
    s: array[0..1000] of char;
begin
  with D^.UserDataPtr^ do begin
    StrPCopy(s, IPAddress + #13#10); _lWrite(AFileHandle, s, StrLen(s));
    StrPCopy(s, IntToStr(RemotePort) + #13#10); _lWrite(AFileHandle, s, StrLen(s));
    for i:=0 to 7 do begin
      StrPCopy(s, IntToStr(InUsed[i]) + #13#10); _lWrite(AFileHandle, s, StrLen(s));
    end;
    for i:=0 to 7 do begin
      StrPCopy(s, IntToStr(OutUsed[i]) + #13#10); _lWrite(AFileHandle, s, StrLen(s));
    end;
  end;
end;

Procedure ReadFromFile(AFileHandle:Word; D: PParameterStruct); export stdcall;
var i: integer;
    s: array[0..1000] of char;
procedure ReadOneLine(FHandle:Word; Aps :PChar);
var i  : Integer;
begin
  i:=0;
  _lRead(FHandle,@Aps[i],1);
  repeat
    inc(i);
    _lRead(FHandle,@Aps[i],1);
  until (Aps[i-1]=#13) and (Aps[i]=#10);
  Aps[i-1]:=#0;
end;
begin
  ReadOneLine(AFileHandle, s); D^.UserDataPtr.IPAddress := StrPas(s);
  ReadOneLine(AFileHandle, s); D^.UserDataPtr.RemotePort := StrToInt(StrPas(s));
  for i:=0 to 7 do begin
    ReadOneLine(AFileHandle, s); D^.UserDataPtr.InUsed[i] := StrToInt(StrPas(s));
  end;
  for i:=0 to 7 do begin
    ReadOneLine(AFileHandle, s); D^.UserDataPtr.OutUsed[i] := StrToInt(StrPas(s));
  end;
end;

procedure IsUserDLL32; export stdcall;
begin
end;

procedure IsDemoDLL; export stdcall;
begin
end;

exports
  GetParameterStruct,
  GetDialogEnableStruct,
  GetNumberOfInputsOutputs,
  CanSimulateDLL,
  InitSimulationDLL,
  SimulateDLL,
  InitUserDLL,
  DisposeUserDLL,
  EndSimulationDLL2,
  GetDLLName,
  CallParameterDialogDLL,
  WriteToFile,
  ReadFromFile,
  SetInputChar,
  SetOutputChar,
  IsDemoDLL,
  IsUserDLL32;


begin
  DecimalSeparator := '.';
  Application.UpdateFormatSettings := false;
end.
